@@ -2,8 +2,10 @@ use crate::*;
 use parking_lot::Mutex;
 use std::{
     any::{Any, TypeId},
-    collections::BTreeMap,
+    cmp::Ordering,
+    collections::{BTreeMap, BTreeSet},
     sync::Arc,
+    thread::spawn,
 };
 
 #[cfg_attr(docsrs, doc(cfg(feature = "app")))]
@@ -29,6 +31,14 @@ pub trait Resource: Sized {
         customizer: impl FnOnce(&mut Self::Customizer, &Self::Config) -> Void,
     ) -> Res<Self>;
 
+    /// Action after initialized and registered to registry.
+    /// [`Factory::init_resource()`] and [`Factory::init_resource_with_builder()`]
+    /// will not call this function, because they don't register
+    /// resource to registry.
+    fn post_initialized_and_registered(_res: &Arc<Self>, _factory: &FactoryContext<'_>) -> Void {
+        Ok(())
+    }
+
     /// Register dependent resources. Create resource will only
     /// request for other resources, if the resource is not
     /// registered yet by [`SalakBuilder::register_resource`],
@@ -48,8 +58,55 @@ pub trait Resource: Sized {
     fn register_dependent_resources(_: &mut FactoryBuilder<'_>) -> Void {
         Ok(())
     }
+
+    /// Order of initializing priority.
+    fn order() -> Ordered {
+        PRIORITY_NORMAL
+    }
 }
 
+/// Resource priority.
+#[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Clone, Copy)]
+pub struct Ordered(i32);
+
+impl Ordered {
+    /// Current resouce should init after then target resouce.
+    pub fn after<R: Resource>(self) -> Self {
+        let r = R::order();
+        match r.cmp(&self) {
+            Ordering::Less => self,
+            Ordering::Equal => Ordered(self.0 + 1),
+            _ => r,
+        }
+    }
+
+    /// Current resouce should init before then target resouce.
+    pub fn before<R: Resource>(self) -> Self {
+        let r = R::order();
+        match r.cmp(&self) {
+            Ordering::Less => r,
+            Ordering::Equal => Ordered(self.0 - 1),
+            _ => self,
+        }
+    }
+}
+
+#[cfg_attr(docsrs, doc(cfg(feature = "app")))]
+/// Highest prioritt.
+pub const PRIORITY_HIGHEST: Ordered = Ordered(i32::MIN + 1048576);
+#[cfg_attr(docsrs, doc(cfg(feature = "app")))]
+/// High prioritt.
+pub const PRIORITY_HIGH: Ordered = Ordered(i32::MIN + 1073741824);
+#[cfg_attr(docsrs, doc(cfg(feature = "app")))]
+/// Normal priority.
+pub const PRIORITY_NORMAL: Ordered = Ordered(0);
+#[cfg_attr(docsrs, doc(cfg(feature = "app")))]
+/// Low priority.
+pub const PRIORITY_LOW: Ordered = Ordered(i32::MAX - 1073741824);
+#[cfg_attr(docsrs, doc(cfg(feature = "app")))]
+/// Lowest priority.
+pub const PRIORITY_LOWEST: Ordered = Ordered(i32::MAX - 1048576);
+
 #[allow(missing_debug_implementations)]
 #[cfg_attr(docsrs, doc(cfg(feature = "app")))]
 /// Context for implementing [`Resource`], which can get dependent
@@ -78,6 +135,7 @@ impl FactoryContext<'_> {
     pub fn get_resource<R: Resource + Send + Sync + Any>(&self) -> Res<Arc<R>> {
         self.get_resource_by_namespace("")
     }
+
     /// Get resource with namespace. The resource will be
     /// initialized if it does not exist yet.
     pub fn get_resource_by_namespace<R: Resource + Send + Sync + Any>(
@@ -92,6 +150,24 @@ impl FactoryContext<'_> {
         );
         self.fac.res.get_ref(namespace, self.fac, false)
     }
+    #[inline]
+    /// Get optional resource.
+    pub fn get_optional_resource<R: Resource + Send + Sync + Any>(&self) -> Res<Option<Arc<R>>> {
+        self.get_optional_resource_by_namespace("")
+    }
+
+    #[inline]
+    /// Get optional resource.
+    pub fn get_optional_resource_by_namespace<R: Resource + Send + Sync + Any>(
+        &self,
+        namespace: &'static str,
+    ) -> Res<Option<Arc<R>>> {
+        match self.get_resource_by_namespace::<R>(namespace) {
+            Ok(v) => Ok(Some(v)),
+            Err(PropertyError::ResourceNotFound(_, _)) => Ok(None),
+            Err(err) => Err(err),
+        }
+    }
 
     /// Get all resouces with same type.
     pub fn get_all_resources<R: Resource + Send + Sync + Any>(
@@ -101,6 +177,29 @@ impl FactoryContext<'_> {
     }
 }
 
+struct Task(
+    Option<
+        Box<
+            dyn FnOnce(&Salak) -> Res<Box<dyn FnOnce() + Send + Sync + 'static>>
+                + Send
+                + Sync
+                + 'static,
+        >,
+    >,
+);
+
+impl Task {
+    fn new<R: Resource + Send + Sync + 'static>(
+        namespace: &'static str,
+        task: impl Fn(Arc<R>) -> Void + Send + Sync + 'static,
+    ) -> Self {
+        Task(Some(Box::new(move |env: &Salak| {
+            let res = env.res.get_ref::<R>(namespace, env, true)?;
+            Ok(Box::new(move || (task)(res).unwrap()))
+        })))
+    }
+}
+
 /// Register dependent resources under same namespace.
 ///
 /// Only relavent resources can be registered by current
@@ -114,6 +213,16 @@ pub struct FactoryBuilder<'a> {
 }
 
 impl FactoryBuilder<'_> {
+    /// Submit remote task
+    pub fn submit<R: Resource + Send + Sync + Any>(
+        &mut self,
+        task: impl Fn(Arc<R>) -> Void + Send + Sync + 'static,
+    ) -> Void {
+        let task = Task::new(self.namespace, task);
+        self.builder.1.push(task);
+        Ok(())
+    }
+
     /// Register dependent resource under current namespace.
     pub fn register_resource<R: Resource + Send + Sync + Any>(&mut self) -> Void {
         self.builder
@@ -207,6 +316,14 @@ pub trait Factory: Environment {
 
     /// Initialize [`Resource`] with builder.
     fn init_resource_with_builder<R: Resource>(&self, builder: ResourceBuilder<R>) -> Res<R>;
+
+    /// Get service.
+    fn get_service<S: Service>(&self) -> Res<S> {
+       self.init_resource::<S>()
+    }
+
+    /// Run the resource.
+    fn run(&mut self) -> Void;
 }
 
 impl Resource for () {
@@ -235,6 +352,10 @@ macro_rules! impl_container {
             ) -> Result<Self, PropertyError> {
                 Ok($x::new(T::create(config, factory, customizer)?))
             }
+
+            fn order() -> Ordered {
+                T::order()
+            }
         }
     )+};
 }
@@ -246,7 +367,7 @@ mod warp {
         rc::Rc,
         sync::{Arc, Mutex, RwLock},
     };
-    impl_container!(Cell RefCell Mutex Rc Arc RwLock);
+    impl_container!(Cell RefCell Mutex RwLock Rc Arc);
 }
 
 impl<T: Resource> Resource for Option<T> {
@@ -264,6 +385,10 @@ impl<T: Resource> Resource for Option<T> {
             Err(e) => Err(e),
         }
     }
+
+    fn order() -> Ordered {
+        T::order()
+    }
 }
 
 struct Init(Box<dyn FnOnce(&Salak, &Mutex<ResVal>) -> Void + Send>);
@@ -272,22 +397,56 @@ impl<R: Resource + Send + Sync + 'static> ResourceBuilder<R> {
     #[inline]
     fn into_init(self) -> Init {
         Init(Box::new(move |env, val| {
-            *val.lock() = Box::new(Arc::new(env.init_resource_with_builder::<R>(self)?));
+            #[cfg(feature = "log")]
+            log::info!(
+                "init resource ({}) at namespace [{}].",
+                std::any::type_name::<R>(),
+                self.namespace
+            );
+            let namespace = self.namespace;
+            let context = FactoryContext {
+                fac: env,
+                namespace,
+            };
+            let res = Arc::new(env.do_init_resource_with_builder::<R>(&context, self)?);
+            R::post_initialized_and_registered(&res, &context)?;
+            *val.lock() = Some(res);
             Ok(())
         }))
     }
 }
 
-type ResVal = Box<dyn Any + Send + Sync>;
+type ResVal = Option<Arc<dyn Any + Send + Sync>>;
 
 /// ResourceHolder is [`Sync`] and [`Send`] only when value in box is [`Send`].
-struct ResourceHolder(Mutex<ResVal>, Mutex<Option<Init>>);
+struct ResourceHolder(Mutex<ResVal>, Mutex<Option<Init>>, Ordered);
+
+impl PartialEq for ResourceHolder {
+    fn eq(&self, r: &ResourceHolder) -> bool {
+        self.2 == r.2
+    }
+}
+
+impl Eq for ResourceHolder {}
+
+impl PartialOrd for ResourceHolder {
+    fn partial_cmp(&self, r: &ResourceHolder) -> Option<Ordering> {
+        self.2.partial_cmp(&r.2)
+    }
+}
+impl Ord for ResourceHolder {
+    fn cmp(&self, r: &Self) -> Ordering {
+        self.2.cmp(&r.2)
+    }
+}
 
 impl ResourceHolder {
     fn new<R: Resource + Send + Sync + 'static>(builder: ResourceBuilder<R>) -> Self {
+        let order = builder.order;
         Self(
-            Mutex::new(Box::new(())),
+            Mutex::new(None),
             Mutex::new(Some(builder.into_init())),
+            order,
         )
     }
 
@@ -308,8 +467,15 @@ impl ResourceHolder {
         query_only: bool,
     ) -> Res<Arc<R>> {
         let guard = self.0.lock();
-        if let Some(val) = guard.downcast_ref::<Arc<R>>() {
-            return Ok(val.clone());
+        if let Some(arc) = guard.as_ref() {
+            if let Ok(v) = arc.clone().downcast::<R>() {
+                return Ok(v);
+            } else {
+                return Err(PropertyError::ResourceNotFound(
+                    namespace,
+                    std::any::type_name::<R>(),
+                ));
+            }
         }
         drop(guard);
         if query_only {
@@ -318,30 +484,36 @@ impl ResourceHolder {
                 std::any::type_name::<R>(),
             ));
         }
-        #[cfg(feature = "log")]
-        log::info!(
-            "Init resource ({}) at namespace [{}].",
-            std::any::type_name::<R>(),
-            namespace
-        );
         self.init(env)?;
-        self.get_or_init(env, namespace, true)
+        match self.get_or_init(env, namespace, true) {
+            Err(PropertyError::ResourceNotFound(a, b)) => {
+                Err(PropertyError::ResourceRecursive(a, b))
+            }
+            v => v,
+        }
     }
 }
 
-pub(crate) struct ResourceRegistry(BTreeMap<TypeId, BTreeMap<&'static str, ResourceHolder>>);
+pub(crate) struct ResourceRegistry(
+    BTreeMap<TypeId, BTreeMap<&'static str, ResourceHolder>>,
+    Vec<Task>,
+);
 
 impl ResourceRegistry {
     pub(crate) fn new() -> Self {
-        Self(BTreeMap::new())
+        Self(BTreeMap::new(), vec![])
     }
 
     pub(crate) fn initialize(&self, env: &Salak) -> Void {
+        let mut v = BTreeSet::new();
         for x in self.0.values() {
             for r in x.values() {
-                r.init(env)?;
+                v.insert(r);
             }
         }
+        for r in v {
+            r.init(env)?;
+        }
         Ok(())
     }
 
@@ -411,6 +583,21 @@ impl ResourceRegistry {
     }
 }
 
+impl Salak {
+    fn do_init_resource_with_builder<R: Resource>(
+        &self,
+        context: &FactoryContext<'_>,
+        builder: ResourceBuilder<R>,
+    ) -> Result<R, PropertyError> {
+        let config = if builder.namespace.is_empty() {
+            self.require::<R::Config>(<R::Config>::prefix())
+        } else {
+            self.require::<R::Config>(&format!("{}.{}", <R::Config>::prefix(), builder.namespace))
+        }?;
+        R::create(config, &context, builder.customizer)
+    }
+}
+
 #[cfg_attr(docsrs, doc(cfg(feature = "app")))]
 impl Factory for Salak {
     #[inline]
@@ -421,20 +608,26 @@ impl Factory for Salak {
         self.res.get_ref(namespace, self, true)
     }
 
+    #[inline]
     fn init_resource_with_builder<R: Resource>(&self, builder: ResourceBuilder<R>) -> Res<R> {
-        let config = if builder.namespace.is_empty() {
-            self.require::<R::Config>(<R::Config>::prefix())
-        } else {
-            self.require::<R::Config>(&format!("{}.{}", <R::Config>::prefix(), builder.namespace))
-        }?;
-        R::create(
-            config,
-            &FactoryContext {
-                fac: self,
-                namespace: builder.namespace,
-            },
-            builder.customizer,
-        )
+        let context = FactoryContext {
+            fac: self,
+            namespace: builder.namespace,
+        };
+        self.do_init_resource_with_builder(&context, builder)
+    }
+
+    fn run(&mut self) -> Void {
+        let mut join = vec![];
+        for mut task in std::mem::replace(&mut self.res.1, vec![]) {
+            if let Some(v) = task.0.take() {
+                join.push(spawn((v)(self)?));
+            }
+        }
+        for join in join {
+            let _ = join.join();
+        }
+        Ok(())
     }
 }
 
@@ -443,6 +636,7 @@ impl Factory for Salak {
 #[allow(missing_debug_implementations)]
 pub struct ResourceBuilder<R: Resource> {
     pub(crate) namespace: &'static str,
+    order: Ordered,
     customizer: Box<dyn FnOnce(&mut R::Customizer, &R::Config) -> Void + Send>,
 }
 
@@ -458,6 +652,7 @@ impl<R: Resource> ResourceBuilder<R> {
     pub fn new(namespace: &'static str) -> Self {
         Self {
             namespace,
+            order: R::order(),
             customizer: Box::new(|_, _| Ok(())),
         }
     }
@@ -480,8 +675,41 @@ impl<R: Resource> ResourceBuilder<R> {
     }
 }
 
+/// Generate a struct which implement as a Service and
+/// provides methods to get reference from this struct.
+///
+///
+#[macro_export]
+macro_rules! generate_service{
+    (@ $name:ident {} -> ($($res:tt)*)) => {
+      #[derive($crate::Service)]
+      #[allow(dead_code,unreachable_pub,missing_debug_implementations, missing_docs)]
+      pub struct $name {
+        $($res)*
+      }
+    };
+    (@ $name:ident {$(#[$m:meta])* $field:ident: Option<$t:ty>, $($tt:tt)*} -> ($($res:tt)*)) => {
+        generate_service!(@ $name {$($tt)*} -> ($($res)* $(#[$m])* $field:Option<Arc<$t>>,));
+    };
+    (@ $name:ident {$(#[$m:meta])* $field:ident: $t:ty, $($tt:tt)*} -> ($($res:tt)*)) => {
+        generate_service!(@ $name {$($tt)*} -> ($($res)* $(#[$m])* $field:Arc<$t>,));
+    };
+    (@ $name:ident {$(#[$m:meta])* $field:ident: Option<$t:ty>} -> ($($res:tt)*)) => {
+        generate_service!(@ $name {} -> ($($res)* $(#[$m])* $field:Option<Arc<$t>>,));
+    };
+    (@ $name:ident { $(#[$m:meta])* $field:ident: $t:ty} -> ($($res:tt)*)) => {
+        generate_service!(@ $name {} -> ($($res)* $(#[$m])* $field:Arc<$t>,));
+    };
+    ($name: ident {$($tt:tt)*}) => {
+        generate_service!(@ $name {$($tt)*} -> ());
+    };
+}
+
+
+
 #[cfg(test)]
 mod tests {
+
     use crate::*;
     #[test]
     fn app_test() {
@@ -496,4 +724,13 @@ mod tests {
         let v = env.get_resource::<()>();
         assert_eq!(true, v.is_ok());
     }
+
+    use std::sync::Arc;
+    generate_service!(X { a: Option<()>, b: ()});
+    generate_service!(Y { a: Option<()>});
+    generate_service!(Z {
+        #[salak(namespace="hello", access="pub")]
+        b:(),
+        a: Option<()>
+    });
 }
